@@ -61,7 +61,13 @@ def _set_output(groups, which):
         output.update(previous)
 
 
-@contextmanager
+def documented_contextmanager(func):
+    wrapper = contextmanager(func)
+    wrapper.undecorated = func
+    return wrapper
+
+
+@documented_contextmanager
 def show(*groups):
     """
     Context manager for setting the given output ``groups`` to True.
@@ -85,7 +91,7 @@ def show(*groups):
     return _set_output(groups, True)
 
 
-@contextmanager
+@documented_contextmanager
 def hide(*groups):
     """
     Context manager for setting the given output ``groups`` to False.
@@ -105,18 +111,25 @@ def hide(*groups):
     return _set_output(groups, False)
 
 
-@contextmanager
-def _setenv(**kwargs):
+@documented_contextmanager
+def _setenv(variables):
     """
     Context manager temporarily overriding ``env`` with given key/value pairs.
 
+    A callable that returns a dict can also be passed. This is necessary when
+    new values are being calculated from current values, in order to ensure that
+    the "current" value is current at the time that the context is entered, not
+    when the context manager is initialized. (See Issue #736.)
+
     This context manager is used internally by `settings` and is not intended
     to be used directly.
     """
-    clean_revert = kwargs.pop('clean_revert', False)
+    if callable(variables):
+        variables = variables()
+    clean_revert = variables.pop('clean_revert', False)
     previous = {}
     new = []
-    for key, value in kwargs.iteritems():
+    for key, value in variables.iteritems():
         if key in state.env:
             previous[key] = state.env[key]
         else:
@@ -126,11 +139,11 @@ def _setenv(**kwargs):
         yield
     finally:
         if clean_revert:
-            for key, value in kwargs.iteritems():
+            for key, value in variables.iteritems():
                 # If the current env value for this key still matches the
                 # value we set it to beforehand, we are OK to revert it to the
                 # pre-block value.
-                if value == state.env[key]:
+                if key in state.env and value == state.env[key]:
                     if key in previous:
                         state.env[key] = previous[key]
                     else:
@@ -150,9 +163,11 @@ def settings(*args, **kwargs):
     * Most usefully, it allows temporary overriding/updating of ``env`` with
       any provided keyword arguments, e.g. ``with settings(user='foo'):``.
       Original values, if any, will be restored once the ``with`` block closes.
+
         * The keyword argument ``clean_revert`` has special meaning for
           ``settings`` itself (see below) and will be stripped out before
           execution.
+
     * In addition, it will use `contextlib.nested`_ to nest any given
       non-keyword arguments, which should be other context managers, e.g.
       ``with settings(hide('stderr'), show('stdout')):``.
@@ -221,7 +236,7 @@ def settings(*args, **kwargs):
     """
     managers = list(args)
     if kwargs:
-        managers.append(_setenv(**kwargs))
+        managers.append(_setenv(kwargs))
     return nested(*managers)
 
 
@@ -315,7 +330,7 @@ def _change_cwd(which, path):
         new_cwd = state.env.get(which) + '/' + path
     else:
         new_cwd = path
-    return _setenv(**{which: new_cwd})
+    return _setenv({which: new_cwd})
 
 
 def path(path, behavior='append'):
@@ -346,7 +361,7 @@ def path(path, behavior='append'):
 
     .. versionadded:: 1.0
     """
-    return _setenv(path=path, path_behavior=behavior)
+    return _setenv({'path': path, 'path_behavior': behavior})
 
 
 def prefix(command):
@@ -401,10 +416,10 @@ def prefix(command):
 
     Contrived, but hopefully illustrative.
     """
-    return _setenv(command_prefixes=state.env.command_prefixes + [command])
+    return _setenv(lambda: {'command_prefixes': state.env.command_prefixes + [command]})
 
 
-@contextmanager
+@documented_contextmanager
 def char_buffered(pipe):
     """
     Force local terminal ``pipe`` be character, not line, buffered.
@@ -420,3 +435,62 @@ def char_buffered(pipe):
             yield
         finally:
             termios.tcsetattr(pipe, termios.TCSADRAIN, old_settings)
+
+
+def shell_env(**kw):
+    """
+    Set shell environment variables for wrapped commands.
+
+    For example, the below shows how you might set a ZeroMQ related environment
+    variable when installing a Python ZMQ library::
+
+        with shell_env(ZMQ_DIR='/home/user/local'):
+            run('pip install pyzmq')
+
+    As with `~fabric.context_managers.prefix`, this effectively turns the
+    ``run`` command into::
+
+        $ export ZMQ_DIR='/home/user/local' && pip install pyzmq
+
+    Multiple key-value pairs may be given simultaneously.
+
+    .. note::
+        If used to affect the behavior of `~fabric.operations.local` when
+        running from a Windows localhost, ``SET`` commands will be used to
+        implement this feature.
+    """
+    return _setenv({'shell_env': kw})
+
+
+quiet = lambda: settings(hide('everything'), warn_only=True)
+quiet.__doc__ = """
+    Alias to ``settings(hide('everything'), warn_only=True)``.
+
+    Useful for wrapping remote interrogative commands which you expect to fail
+    occasionally, and/or which you want to silence.
+
+    Example::
+
+        with quiet():
+            have_build_dir = run("test -e /tmp/build").succeeded
+
+    When used in a task, the above snippet will not produce any ``run: test -e
+    /tmp/build`` line, nor will any stdout/stderr display, and command failure
+    is ignored.
+
+    .. seealso::
+        :ref:`env.warn_only <warn_only>`,
+        `~fabric.context_managers.settings`,
+        `~fabric.context_managers.hide`
+"""
+
+
+warn_only = lambda: settings(warn_only=True)
+warn_only.__doc__ = """
+    Alias to ``settings(warn_only=True)``.
+
+    .. seealso::
+        :ref:`env.warn_only <warn_only>`,
+        `~fabric.context_managers.settings`,
+        `~fabric.context_managers.quiet`
+"""
