@@ -13,7 +13,7 @@ import socket
 import sys
 
 from fabric.auth import get_password, set_password
-from fabric.utils import abort, handle_prompt_abort
+from fabric.utils import abort, handle_prompt_abort, warn
 from fabric.exceptions import NetworkError
 
 try:
@@ -136,8 +136,9 @@ def ssh_config(host_string=None):
     May give an explicit host string as ``host_string``.
     """
     from fabric.state import env
+    dummy = {}
     if not env.use_ssh_config:
-        return {}
+        return dummy
     if '_ssh_config' not in env:
         try:
             conf = ssh.SSHConfig()
@@ -146,7 +147,8 @@ def ssh_config(host_string=None):
                 conf.parse(fd)
                 env._ssh_config = conf
         except IOError:
-            abort("Unable to load SSH config file '%s'" % path)
+            warn("Unable to load SSH config file '%s'" % path)
+            return dummy
     host = parse_host_string(host_string or env.host_string)['host']
     return env._ssh_config.lookup(host)
 
@@ -167,10 +169,12 @@ def key_filenames():
     # Strip out any empty strings (such as the default value...meh)
     keys = filter(bool, keys)
     # Honor SSH config
-    # TODO: fix ssh so it correctly treats IdentityFile as a list
     conf = ssh_config()
     if 'identityfile' in conf:
-        keys.append(conf['identityfile'])
+        # Handle Paramiko 1.9 vs 1.10 treatment of this setting.
+        # 1.9 is a string, 1.10 is a list of strings.
+        value = conf['identityfile']
+        keys.extend([value] if isinstance(value, basestring) else value)
     return map(os.path.expanduser, keys)
 
 
@@ -459,6 +463,11 @@ def connect(user, host, port, sock=None):
                 sock.close()
 
 
+def _password_prompt(prompt, stream):
+    # NOTE: Using encode-to-ascii to prevent (Windows, at least) getpass from
+    # choking if given Unicode.
+    return getpass.getpass(prompt.encode('ascii', 'ignore'), stream)
+
 def prompt_for_password(prompt=None, no_colon=False, stream=None):
     """
     Prompts for and returns a new password if required; otherwise, returns
@@ -485,12 +494,12 @@ def prompt_for_password(prompt=None, no_colon=False, stream=None):
     if not no_colon:
         password_prompt += ": "
     # Get new password value
-    new_password = getpass.getpass(password_prompt, stream)
+    new_password = _password_prompt(password_prompt, stream)
     # Otherwise, loop until user gives us a non-empty password (to prevent
     # returning the empty string, and to avoid unnecessary network overhead.)
     while not new_password:
         print("Sorry, you can't enter an empty password. Please try again.")
-        new_password = getpass.getpass(password_prompt, stream)
+        new_password = _password_prompt(password_prompt, stream)
     return new_password
 
 
